@@ -2,9 +2,7 @@ from FrozenYoghourt import *
 from FrozenYoghourt.mode import *
 from FrozenYoghourt.maths import *
 
-
-
-def Id():
+def ID():
     if Mode.representation == 'numpy':
         return np.array([[1, 0], [0, 1]])
     else:
@@ -27,6 +25,50 @@ def Z():
         return np.array([[1, 0], [0, -1]])
     else:
         return Matrix([[1, 0], [0, -1]])
+    
+def pauli(val, mul = 1):
+    
+    """
+    Return a Pauli matrix or tensor product of matrices
+    
+    Parameter
+    ---------
+    val: int, str, list
+        Value or list of values of matrices in either integer (0, 1, 2, 3)
+        or string ('i', 'x', 'y', 'z') form. If a list is given, compute 
+        the tensor product of the pauli's in the list from left to right.
+    
+    mul: int
+        Multiplicity of value or list of values
+        
+    Returns
+    -------
+    pauli_mat: np.ndarray, sympy.Matrices
+        Pauli matrix or tensor product of matrices
+        
+    """
+    
+    option = type(val)
+    if option == list:
+        option = type(val[0])
+    
+    if option == int:
+        num = {0: ID(), 1: X(), 2: Y(), 3:Z()}
+        if type(val) == int:
+            pauli_mat = tp(num[val], mul = mul)
+        else:
+            pauli_list = [num[elem] for elem in val]
+            pauli_mat = tp(*pauli_list, mul = mul)
+      
+    elif option == str:
+        text = {'i': ID(), 'x': X(), 'y': Y(), 'z': Z()}
+        if type(val) == str:
+            pauli_mat = tp(text[val], mul = mul)
+        else:
+            pauli_list = [text[elem] for elem in val]
+            pauli_mat = tp(*pauli_list, mul = mul)
+            
+    return pauli_mat
 
 def H():
     if Mode.representation == 'numpy':
@@ -58,19 +100,34 @@ def Rz(theta:float):
         return Matrix([[exp(-I * theta / 2), 0],
                        [0, exp(I * theta / 2)]])
 
-def P(theta:float):
+def Phase(theta:float):
     if Mode.representation == 'numpy':
         return np.array([[1, 0], [0, np.exp(1j*theta)]])
     else:
         return Matrix([[1, 0], [0, exp(I*theta)]])
 
-def U(theta, phi, lam):
+def u2(angles:list = None, unimodular = False, index = ''):
+    
+    if type(angles) == bool:
+        unimodular = angles
+        angles = None
+    
     if Mode.representation == 'numpy':
-        return np.array([[np.cos(theta / 2), -np.exp(1j * lam) * np.sin(theta / 2)],
-                         [np.exp(1j * phi) * np.sin(theta / 2), np.exp(1j * (phi + lam) * np.cos(theta / 2))]])
+        if unimodular: 
+            return to_su(random_unitary(2).data)
+        else: 
+            return random_unitary(2).data
     else:
-        return Matrix([[cos(theta / 2), -exp(I * lam) * sin(theta / 2)],
-                       [exp(I * phi) * sin(theta / 2), exp(I * (phi + lam)) * cos(theta / 2)]])
+        if angles is None:
+            theta, phi, lam = symbols(f'theta_{index}, phi_{index}, lambda_{index}', real = True)
+        else:
+            theta, phi, lam = angles
+        if unimodular:
+            return Matrix([[exp(I*phi)*cos(theta / 2), -exp(I * -lam) * sin(theta / 2)],
+                                          [exp(I * lam) * sin(theta / 2), exp(I * (-phi)) * cos(theta / 2)]])
+        else:
+            return Matrix([[cos(theta / 2), -exp(I * lam) * sin(theta / 2)],
+                                           [exp(I * phi) * sin(theta / 2), exp(I * (phi + lam)) * cos(theta / 2)]])
 
 def CX():
     if Mode.representation == 'numpy':
