@@ -1,13 +1,14 @@
 from FrozenYoghourt import *
 from FrozenYoghourt.mode import *
 
+close = lambda a, b: np.all(np.isclose(a, b))
 
-def mm(*lst, no_times=1):
-    lst *= no_times
+def mm(*lst, mult=1):
+    lst *= mult
     return reduce(lambda a, b: a @ b, lst)
 
-def tp(*lst, no_times=1):
-    lst *= no_times
+def tp(*lst, mult=1):
+    lst *= mult
     if Mode.representation == 'numpy':
         return reduce(lambda a, b: np.kron(a, b), lst)
     else:
@@ -72,17 +73,21 @@ def to_su(*u:Union[Tuple[np.ndarray], Tuple[MutableDenseMatrix]]):
             return to_su_list
     else:
         if len(u) == 1:
-            return u[0] * complex(u[0].det()) ** (-1 / u[0].shape[0])
+            return (u[0] * complex(u[0].det()) ** (-1 / u[0].shape[0])).evalf()
         else:
-            to_su_list = [mat * complex(mat.det()) ** (-1 / mat.shape[0]) for mat in u]
+            to_su_list = [(mat * complex(mat.det()) ** (-1 / mat.shape[0])).evalf() for mat in u]
             return to_su_list
 
 def fast_substitution(matrix:Matrix, variables, values, to_numpy=False):
-    substituted_matrix = matrix.subs(list(zip(variables, values)))
-    if to_numpy:
-        return np.array(substituted_matrix).astype(np.complex)
+    if (type(variables) == list) or (type(variables) == tuple):
+        substituted_matrix = matrix.subs(list(zip(variables, values)))
     else:
-        return substituted_matrix
+        substituted_matrix = matrix.subs(variables, values)
+        
+    if to_numpy:
+        return np.array(substituted_matrix).astype(complex)
+    else:
+        return substituted_matrix.evalf()
 
 def optimizing_identities(lhs_fun, rhs_fun, no_var, var_split, random=(0, 2 * np.pi), num_qubits=2, error=1e-7):
     compute_rhs = True
